

this.TCar=function(model, wheelmodel, weight, wheelWeight, startpos)
{
    this.initTCar=function(model, wheelmodel, weight, wheelWeight, startpos)
    {
        this.model=Model(model).add();
        this.model.position=startpos;
        this.wheelmodelName=wheelmodel;
        this.weight=weight;
        this.wheelWeight=wheelWeight;

        this.wheelsCreated=false;
        
        this.fixedWheels=new Array();
        this.steerWheels=new Array();
        this.solver=undefined;
        
        addAnimator(this);
    }
    this.initTCar(model, wheelmodel, weight, wheelWeight, startpos);
    
    this.timestep=function(t, dt)
    {
    	var wheelAnchors, i, wheel;

        if(!this.model.loaded)
            return;

        if(!this.wheelsCreated)
        {
            // Do fixed wheels first.
            wheelAnchors=this.model.getSolidObject(0).rootPrimitive.findAll(".../wheel");

            for(i=0;i<wheelAnchors.length;i++)
            {
                wheel=new this.TWheel(this.wheelmodelName,
                    this.model.getSolidObject(0).mapLocalTransformToWorldTransform(Transform(wheelAnchors[i].position)).translation,
                    this.wheelWeight);

                this.fixedWheels.push(wheel);
            }

            // Do steer wheels next.
            wheelAnchors=this.model.getSolidObject(0).rootPrimitive.findAll(".../wheelsteer");

            for(i=0;i<wheelAnchors.length;i++)
            {
                wheel=new this.TWheel(this.wheelmodelName,
                    this.model.getSolidObject(0).mapLocalTransformToWorldTransform(Transform(wheelAnchors[i].position)).translation,
                    this.wheelWeight);

                this.steerWheels.push(wheel);
            }

            this.wheelsCreated=true;
            
            return;
        }

        // Check if all wheels are done loading.
        for(i=0;i<this.fixedWheels.length;i++)
        {
            if(!this.fixedWheels[i].loaded)
                return;
        }

        for(i=0;i<this.steerWheels.length;i++)
        {
            if(!this.steerWheels[i].loaded)
                return;
        }
        
        // Set up a physicalModel for the car itself.
        this.model.physicalModel = this.model.getSolidObject(0).createPhysicalModel(this.weight);
        this.model.physicalModel.collide=true;
        this.model.physicalModel.acceleration=Vector(0, -32, 0);

        // Create a constraints solver.
        this.solver=FastConstraintSolver();

        // Set up all wheel constraints.
        for(i=0;i<this.fixedWheels.length;i++)
        {
            this.fixedWheels[i].setupConstraint(this.solver, this.model.physicalModel);
        }

        for(i=0;i<this.steerWheels.length;i++)
        {
            this.steerWheels[i].setupConstraint(this.solver, this.model.physicalModel);
        }

        // We are done loading the car, stop this timestep function.
        removeAnimator(this);
    }
    
    this.setSteeringAngle=function(angle)
    {
    	var i;
    	
    	for(i=0;i<this.steerWheels.length;i++)
    	{
    	    this.steerWheels[i].setSteeringAngle(angle);
    	}
    }

    this.setSpeedAndMaxTorque=function(speed, torque)
    {
    	var i;

    	for(i=0;i<this.fixedWheels.length;i++)
    	{
    	    this.fixedWheels[i].setSpeedAndMaxTorque(speed, torque);
    	}
    	for(i=0;i<this.steerWheels.length;i++)
    	{
    	    this.steerWheels[i].setSpeedAndMaxTorque(speed, torque);
    	}
    }

    this.getOrientation=function()
    {
    	if(this.model.physicalModel)
            return this.model.physicalModel.orientation;
            
        return this.model.orientation;
    }
    
    this.getPosition=function()
    {
    	if(this.model.physicalModel)
    	{
    	    return this.model.physicalModel.position;
    	}

        return this.model.position;
    }
}
this.TCar.prototype.TWheel=function(model, position, weight)
{
    this.initTWheel=function(model, position, weight)
    {
    	this.model=Model(model).add();
    	this.model.position=position;
    	
    	this.weight=weight;
    	this.loaded=false;
    	
    	addAnimator(this);
    }
    this.initTWheel(model, position, weight);
    
    this.timestep=function(t, dt)
    {
    	if(!this.model.loaded)
    	    return;
    	    
    	this.physicalModel=this.model.getSolidObject(0).createPhysicalModel(this.weight);
    	
    	this.loaded=true;
    	removeAnimator(this);

    	return;
    }
    
    this.setupConstraint=function(solver, carbody)
    {
        this.constraint=solver.createWheelConstraint(carbody, this.physicalModel);
        this.constraint.strength=1;
        this.constraint.tau=1;
        this.constraint.setWorldSpacePivotAxleAndSuspension(
            this.model.position,
            Vector(1, 0, 0),
            Vector(0, 1, 0)
        );
        solver.addConstraint(this.constraint);

        this.physicalModel.collide=true;
        this.physicalModel.inertiaScale=1;
        this.physicalModel.acceleration=Vector(0, -10, 0);
        this.physicalModel.friction=250;
    }
    
    this.setSteeringAngle=function(angle)
    {
    	if(this.constraint)
            this.constraint.setSteeringAngle(angle);
    }

    this.setSpeedAndMaxTorque=function(speed, torque)
    {
    	if(this.constraint)
    	    this.constraint.setMotorSpeedAndMaxTorque(speed, torque);
    }
}





this.TCarInterface=function(car)
{
    this.initTCarInterface=function(car)
    {
    	this.car=car;
    }
    this.initTCarInterface(car);

    this.enable=function()
    {
    	addAnimator(this);

        actor.keyForwardBackwardMovesEnabled=false;
        actor.keyLeftRightMovesEnabled=false;
        actor.keyLeftRightTurnsEnabled=false;
        actor.keyUpDownMovesEnabled=false;
        actor.keyUpDownTurnsEnabled=false;
    }

    this.disable=function()
    {
    	removeAnimator(this);

        actor.keyForwardBackwardMovesEnabled=true;
        actor.keyLeftRightMovesEnabled=true;
        actor.keyLeftRightTurnsEnabled=true;
        actor.keyUpDownMovesEnabled=true;
        actor.keyUpDownTurnsEnabled=true;
    }
    
    this.timestep=function(t, dt)
    {
        actor.keyForwardBackwardMovesEnabled=false;
        actor.keyLeftRightMovesEnabled=false;
        actor.keyLeftRightTurnsEnabled=false;
        actor.keyUpDownMovesEnabled=false;
        actor.keyUpDownTurnsEnabled=false;
    
        if(application.leftArrowKeyDown)
            this.car.setSteeringAngle(Math.PI/-3);
        else if(application.rightArrowKeyDown)
            this.car.setSteeringAngle(Math.PI/3);
        else
            this.car.setSteeringAngle(0);

        if(application.upArrowKeyDown)
            this.car.setSpeedAndMaxTorque(50, 150);
        else if(application.downArrowKeyDown)
            this.car.setSpeedAndMaxTorque(-25, 100);
        else
            this.car.setSpeedAndMaxTorque(0, 15);
    }
}


this.TDisplayControl=function(car)
{
    this.initTDisplayControl=function(car)
    {
    	this.car=car;
    }
    this.initTDisplayControl(car);
    
    this.lookAt=function(pos)
    {
    	// As in the API docs.
    	var deltaPos, actorRange, actorRangeXZ, actorPitch, actorYaw, targetOrientation;

        deltaPos      = actor.position.subtract(pos);
        
        actorRange   = Math.sqrt(deltaPos.x*deltaPos.x +
                                 deltaPos.y*deltaPos.y +
                                 deltaPos.z*deltaPos.z);
        actorRangeXZ = Math.sqrt(deltaPos.x*deltaPos.x +
                                 deltaPos.z*deltaPos.z);
        actorPitch   = Math.atan2(-deltaPos.y, actorRangeXZ);
        actorYaw     = Math.atan2(deltaPos.x, deltaPos.z);
        
        targetOrientation = Rotation('YX', actorYaw, actorPitch);
        actor.alignHeadAndBodyTo(targetOrientation);
    }
    
    this.enable=function()
    {
    	addAnimator(this);
    }
    
    this.disable=function()
    {
    	removeAnimator(this);
    }
    
    this.timestep=function(t, dt)
    {
    	this.lookAt(this.car.getPosition());
    }
}




//this.car=new this.genericCar("car.aer", "wheel.aer", 25, 1);
this.car=new this.TCar("car.aer", "wheel.aer", 25, 1, this.position);

this.carInterface=new this.TCarInterface(this.car);
this.carInterface.enable();

this.displayControl=new this.TDisplayControl(this.car);
this.displayControl.enable();



